@@ -1,5 +1,6 @@
 from celery.decorators import task, periodic_task
 from celery.schedules import crontab
+from .utils import filelock
 
 from .settings import RECOMMENDS_TASK_RUN, RECOMMENDS_TASK_CRONTAB
 
@@ -14,7 +15,9 @@ if RECOMMENDS_TASK_RUN:
         def _precompute(provider_instance):
             prefs = provider_instance.prefs()
             provider_instance.precompute(prefs)
-        [_precompute(provider_instance) for model, provider_instance in recommendation_registry.providers.iteritems()]
+
+        with filelock('recommends_precompute.lock'):
+            [_precompute(provider_instance) for model, provider_instance in recommendation_registry.providers.iteritems()]
 
 
 @task
@@ -30,3 +33,16 @@ def remove_suggestion(user_id, rating_model, model_path, object_id):
     obj = ObjectClass.objects.get(pk=object_id)
 
     provider_instance.storage.remove_recommendation(user, obj)
+
+
+@task
+def remove_similarity(rating_model, model_path, object_id):
+    from django.db.models import get_model
+    from recommends.providers import recommendation_registry
+
+    provider_instance = recommendation_registry.providers[rating_model]
+
+    ObjectClass = get_model(*model_path.split('.'))
+    obj = ObjectClass.objects.get(pk=object_id)
+
+    provider_instance.storage.remove_similarity(obj)
